@@ -154,9 +154,10 @@ function send_pick(cvs, p, seg) {
 	events.push(evt)
 }
 
-function send_compound(cvs, pos, size) {
+function send_compound(cvs, pos, size, up) {
     "use strict";
-    var evt = {event: '_compound', 'canvas': cvs, 'pos': [pos.x, pos.y, pos.z], 'size': [size.x, size.y, size.z]}
+    var evt = {event: '_compound', 'canvas': cvs, 'pos': [pos.x, pos.y, pos.z], 
+        'size': [size.x, size.y, size.z], 'up': [up.x, up.y, up.z]}
 	events.push(evt)
 }
 
@@ -241,7 +242,7 @@ function process_waitfor(evt) {
 }
 
 // attrs are X in {'a': '23X....'}
-var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use single and double quotes; available: +-, 
+var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use single and double quotes; available: -, 
          'e':'ambient', 'f':'axis', 'g':'size', 'h':'origin', 'i':'textcolor',
          'j':'direction', 'k':'linecolor', 'l':'bumpaxis', 'm':'dot_color',
          'n':'foreground', 'o':'background', 'p':'ray', 'E':'center', '#':'forward', '+':'resizable',
@@ -273,14 +274,15 @@ var attrsb = {'a':'userzoom', 'b':'userspin', 'c':'range', 'd':'autoscale', 'e':
               'p':'left', 'q':'right', 'r':'top', 's':'bottom', 't':'_cloneid'}
 
 // methods are X in {'m': '23X....'}
-var methods = {'a':'select', 'b':'pos', 'c':'start', 'd':'stop', 'f':'clear', // unused eghijklmnopuvxyzCDFAB
+var methods = {'a':'select', 'b':'pos', 'c':'start', 'd':'stop', 'f':'clear', // unused eghijklmnopvxyzCDFAB
 			   'q':'plot', 's':'add_to_trail',
-               't':'follow', 'w':'clear_trail',
+               't':'follow', 'u':'_attach_arrow', 'w':'clear_trail',
                'G':'bind', 'H':'unbind', 'I':'waitfor', 'J':'pause', 'K':'pick', 'L':'GSprint',
 		       'M':'delete'}
          
 var vecattrs = ['pos', 'up', 'color', 'trail_color', 'axis', 'size', 'origin', 'textcolor',
                 'direction', 'linecolor', 'bumpaxis', 'dot_color', 'ambient', 'add_to_trail',
+                '_attach_arrow',
                 'foreground', 'background', 'ray', 'ambient', 'center', 'forward', 'normal']
                 
 var textattrs = ['text', 'align', 'caption', 'title', 'xtitle', 'ytitle', 'selected',
@@ -356,9 +358,16 @@ function decode(data) {
 function fix_location(cfgx) {
     "use strict";
     if ('location' in cfgx) {
-        if (cfgx['location'] === 1) {cfgx['pos'] = cfgx['canvas'].title_anchor}
-        else if (cfgx['location'] === 2) {cfgx['pos'] = cfgx['canvas'].caption_anchor}
-        else if (cfgx['location'] === 3) {cfgx['pos'] = print_anchor}
+        var loc = cfgx['location']
+        var id = loc[0]
+        if (id == -1) {
+            cfgx['pos'] = print_anchor // this doesn't work; throw an error in vpython.py
+        } else {
+            var cvs = glowObjs[id]
+            var where = loc[1]
+            if (where === 1) cfgx['pos'] = cvs.title_anchor
+            else cfgx['pos'] = cvs.caption_anchor
+        }
         delete cfgx['location']
     }
     return cfgx
@@ -409,7 +418,7 @@ function handle_cmds(dcmds) {
 
 		//assembling cfg
 		var vlst = ['pos', 'color', 'size', 'axis', 'up', 'direction', 'center', 'forward', 'foreground',
-				 'background', 'ambient', 'linecolor', 'dot_color', 'trail_color', 'textcolor',
+				 'background', 'ambient', 'linecolor', 'dot_color', 'trail_color', 'textcolor', 'attrval',
 				 'origin', 'normal', 'bumpaxis','texpos', 'start_face_color', 'end_face_color']
 		if ((obj != 'gcurve') && ( obj != 'gdots' ) ) vlst.push( 'size' )
 		var cfg = {}
@@ -537,14 +546,14 @@ function handle_cmds(dcmds) {
 				} else {
 					var obj = glowObjs[idx] = compound(objects, cfg)
 					// Return computed compound pos and size to Python
-					send_compound(obj.canvas['idx'], obj.pos, obj.size)
+					send_compound(obj.canvas['idx'], obj.pos, obj.size, obj.up)
 				}
 				break
 			}
 			case 'extrusion': {
 				var obj = glowObjs[idx] = extrusion(cfg)
 				// Return computed compound pos and size to Python
-				send_compound(obj.canvas['idx'], obj.pos, obj.size)
+				send_compound(obj.canvas['idx'], obj.pos, obj.size, obj.up)
 				break
 			}
 			case 'text':     {
@@ -555,7 +564,8 @@ function handle_cmds(dcmds) {
 				} else {
 					// Return text parameters to Python
 					var obj = glowObjs[idx] = text(cfg)
-					send_compound(obj.canvas['idx'], vec(obj.length, obj.descender, 0), vec(0,0,0))
+					send_compound(obj.canvas['idx'], vec(obj.length, obj.descender, 0), 
+                            obj.__comp.size, obj.up)
 				}
 				break
 			}
@@ -569,10 +579,15 @@ function handle_cmds(dcmds) {
 					//$("<div id='fps'/>").appendTo(glowObjs[idx].title)
 			}
 			case 'attach_arrow':  {
-				var o = glowObjs[cfg['_obj']]
-				delete cfg['_o']
-				var attr = cfg['_attr']
-				delete cfg['_attr']
+                var attrs = ['pos', 'size', 'axis', 'up', 'color']
+				var o = glowObjs[cfg['obj']]
+				delete cfg['obj']
+				var attr = cfg['attr']
+				delete cfg['attr']
+                var val = cfg['attrval']
+                delete cfg['attrval']
+                if (attrs.indexOf(attr) < 0) attr = '_attach_arrow'
+                o.attr = val
 				glowObjs[idx] = attach_arrow( o, attr, cfg )
 				break
 			}
@@ -586,6 +601,12 @@ function handle_cmds(dcmds) {
 				glowObjs[idx] = attach_trail(o, cfg)
 				break
 			}
+            case 'wtext': {
+				cfg.objName = obj
+				cfg = fix_location(cfg)
+				glowObjs[idx] = wtext(cfg)
+				break
+            }
 			case 'checkbox': {
 				cfg.objName = obj
 				cfg.bind = control_handler
@@ -693,6 +714,8 @@ function handle_methods(dmeth) {
 			glowObjs[idx]['pos'] = val
 		} else if (method === 'add_to_trail') {
 			obj['_func'] = val
+		} else if (method === '_attach_arrow') {
+            obj.obj._attach_arrow = val
 		} else if (method === 'bind') {
 			var evts = val.split(' ')
 			for (var evt in evts) {
